package meerkat.crypto.dkg.comm;

import com.google.protobuf.InvalidProtocolBufferException;
import com.google.protobuf.Message;
import meerkat.crypto.utils.Channel;
import meerkat.protobuf.DKG;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created by Tzlil on 2/14/2016.
 *
 * an implementation of ReceiverCallback
 */
public abstract class MailHandler implements Channel.ReceiverCallback{
    final Logger logger = LoggerFactory.getLogger(getClass());

    /**
     * fixed value for broadcasting
     */
    public static final int BROADCAST = 0;

    /**
     * message handler
     */
    private MessageHandler messageHandler;

    /**
     * constructor
     * @param messageHandler
     */
    public MailHandler(MessageHandler messageHandler){
        this.messageHandler = messageHandler;
    }

    /**
     * Was this broadcastMessage was received by broadcast channel
     * @param broadcastMessage
     * @return broadcastMessage user destination == BROADCAST
     */
    public boolean isBroadcast(DKG.BroadcastMessage broadcastMessage){
        return broadcastMessage.getDestination() == BROADCAST;
    }

    @Override
    public void receiveMail(DKG.BroadcastMessage envelope) {
        try {
            messageHandler.handleMessage(envelope);
        } catch (InvalidProtocolBufferException e) {
            logger.warn("Received invalid protocol buffer from channel", e);
        }
    }
}
