package meerkat.crypto.utils;

import com.google.protobuf.Message;
import meerkat.protobuf.DKG;

import java.util.Queue;
import java.util.concurrent.ArrayBlockingQueue;

/**
 * Created by Tzlil on 2/14/2016.
 */
// TODO: Change nane to network

public class ChannelImpl implements Channel {

    public static int BROADCAST = 0;
    private static ChannelImpl[] channels = null;

    protected final Queue<DKG.BroadcastMessage> mailbox;
    protected final int id;
    protected final int n;
    protected Thread receiverThread;


    public ChannelImpl(int id, int n) {
        if (channels == null){
            channels = new ChannelImpl[n];
        }
        this.mailbox = new ArrayBlockingQueue<DKG.BroadcastMessage>( n * n * n);
        this.id = id;
        this.n = n;
        channels[id - 1] = this;
    }

    @Override
    public int getSourceId() {
        return id;
    }


    @Override
    public void sendMessage(int destUser, Message msg) {
        if(destUser < 1 || destUser > n)
            return;
        ChannelImpl channel = channels[destUser - 1];
        if (channel == null)
            return;
        DKG.BroadcastMessage broadcastMessage = DKG.BroadcastMessage.newBuilder()
                .setSender(id)
                .setDestination(destUser)
                .setIsPrivate(true)
                .setPayload(msg.toByteString())
                .build();
        synchronized (channel.mailbox) {
            channel.mailbox.add(broadcastMessage);
            channel.mailbox.notify();
        }
    }

    @Override
    public void broadcastMessage(Message msg) {
        ChannelImpl channel;
        DKG.BroadcastMessage broadcastMessage = DKG.BroadcastMessage.newBuilder()
                .setSender(id)
                .setDestination(BROADCAST)
                .setIsPrivate(false)
                .setPayload(msg.toByteString())
                .build();
        for (int i = 0 ; i < n ; i++){
            channel = channels[i];
            synchronized (channel.mailbox) {
                channel.mailbox.add(broadcastMessage);
                channel.mailbox.notify();
            }
        }
    }

    public void stop(){
        try{
            receiverThread.interrupt();
        }catch (Exception e){
            //do nothing
        }
    }

    @Override
    public void registerReceiverCallback(final ReceiverCallback callback) {
        stop();
        receiverThread = new Thread(new Runnable() {
            @Override
            public void run() {
                while (true){
                    try {
                        synchronized (mailbox) {
                            while (!mailbox.isEmpty()) {
                                callback.receiveMail(mailbox.remove());
                            }
                            mailbox.wait();
                        }
                    } catch (InterruptedException e) {
                        //do nothing
                    }
                }
            }
        });
        receiverThread.start();
    }



}
